@@ -1,38 +1,6 @@
-# Copyright (c) 2009-2010 Aldo Cortesi
-# Copyright (c) 2010 matt
-# Copyright (c) 2010, 2012, 2014 dequis
-# Copyright (c) 2010 Philip Kranz
-# Copyright (c) 2010-2011 Paul Colomiets
-# Copyright (c) 2011 osebelin
-# Copyright (c) 2011 Mounier Florian
-# Copyright (c) 2011 Kenji_Takahashi
-# Copyright (c) 2011 Tzbob
-# Copyright (c) 2012, 2014 roger
-# Copyright (c) 2012, 2014-2015 Tycho Andersen
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2014-2015 Sean Vig
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 """
-    A minimal EWMH-aware OO layer over xcffib. This is NOT intended to be
-    complete - it only implements the subset of functionalty needed by qtile.
+A minimal EWMH-aware OO layer over xcffib. This is NOT intended to be
+complete - it only implements the subset of functionalty needed by qtile.
 """
 
 from __future__ import annotations
@@ -254,7 +222,7 @@ class MaskMap:
                     values.append(getattr(val, "_maskvalue", val))
                 del kwargs[s]
         if kwargs:
-            raise ValueError("Unknown mask names: %s" % list(kwargs.keys()))
+            raise ValueError(f"Unknown mask names: {list(kwargs.keys())}")
         return mask, values
 
 
@@ -405,10 +373,28 @@ class RandR:
 
     def query_crtcs(self, root):
         infos = []
-        for crtc in self.ext.GetScreenResources(root).reply().crtcs:
-            crtc_info = self.ext.GetCrtcInfo(crtc, xcffib.CurrentTime).reply()
+        primary = self.ext.GetOutputPrimary(root).reply().output
+        for output in self.ext.GetScreenResources(root).reply().outputs:
+            info = self.ext.GetOutputInfo(output, xcffib.CurrentTime).reply()
 
-            infos.append(ScreenRect(crtc_info.x, crtc_info.y, crtc_info.width, crtc_info.height))
+            # ignore disconnected monitors
+            if info.connection != xcffib.randr.Connection.Connected:
+                continue
+
+            # ignore outputs with no monitor plugged in
+            if not info.crtc:
+                continue
+
+            crtc_info = self.ext.GetCrtcInfo(info.crtc, xcffib.CurrentTime).reply()
+
+            rect = ScreenRect(crtc_info.x, crtc_info.y, crtc_info.width, crtc_info.height)
+
+            # prepend the primary output, append all others in screen
+            # resources order
+            if primary == output:
+                infos.insert(0, rect)
+            else:
+                infos.append(rect)
         return infos
 
     def enable_screen_change_notifications(self, conn):
@@ -483,7 +469,9 @@ class Connection:
 
     @property
     def pseudoscreens(self):
-        if hasattr(self, "xinerama"):
+        if hasattr(self, "randr"):
+            return self.randr.query_crtcs(self.screens[0].root.wid)
+        elif hasattr(self, "xinerama"):
             pseudoscreens = []
             for i, s in enumerate(self.xinerama.query_screens()):
                 scr = ScreenRect(
@@ -494,10 +482,14 @@ class Connection:
                 )
                 pseudoscreens.append(scr)
             return pseudoscreens
-        elif hasattr(self, "randr"):
-            return self.randr.query_crtcs(self.screens[0].root.wid)
+        raise Exception("no randr or xinerama?")
 
     def enable_screen_change_notifications(self):
+        if not hasattr(self, "randr"):
+            logger.warning(
+                "no randr configured for this X server, screen change notifications disabled"
+            )
+            return
         self.randr.enable_screen_change_notifications(self)
 
     def finalize(self):
@@ -618,7 +610,7 @@ class Connection:
                 xcffib.xproto.Time.CurrentTime,
             )
 
-    @functools.lru_cache()
+    @functools.lru_cache
     def color_pixel(self, name):
         pixel = self.screens[0].default_colormap.alloc_color(name).pixel
         return pixel | 0xFF << 24
@@ -642,8 +634,8 @@ class Painter:
         # necessary size of the root window by looking at the
         # pseudoscreens attribute and calculating the max x and y extents.
         root_windows = screen.qtile.core.conn.pseudoscreens
-        width = max((win.x + win.width for win in root_windows))
-        height = max((win.y + win.height for win in root_windows))
+        width = max(win.x + win.width for win in root_windows)
+        height = max(win.y + win.height for win in root_windows)
 
         try:
             root_pixmap = self.default_screen.root.get_property(
@@ -728,7 +720,7 @@ class Painter:
         try:
             with open(image_path, "rb") as f:
                 image, _ = cairocffi.pixbuf.decode_to_image_surface(f.read())
-        except IOError:
+        except OSError:
             logger.exception("Could not load wallpaper:")
             return
 
@@ -749,12 +741,19 @@ class Painter:
                     height_ratio = screen.height / image_h
                     context.translate(-(image_w * height_ratio - screen.width) // 2, 0)
                     context.scale(height_ratio)
+                context.set_source_surface(image)
             elif mode == "stretch":
                 context.scale(
                     sx=screen.width / image.get_width(),
                     sy=screen.height / image.get_height(),
                 )
-            context.set_source_surface(image)
+                context.set_source_surface(image)
+            elif mode == "center":
+                target_x = (screen.width - image.get_width()) / 2
+                target_y = (screen.height - image.get_height()) / 2
+                context.set_source_surface(image, x=target_x, y=target_y)
+            else:
+                context.set_source_surface(image)
             context.paint()
 
         surface.finish()
@@ -767,7 +766,7 @@ class Painter:
 def get_keysym(key: str) -> int:
     keysym = keysyms.get(key.lower())
     if not keysym:
-        raise XCBQError("Unknown key: %s" % key)
+        raise XCBQError(f"Unknown key: {key}")
     return keysym
 
 
@@ -789,7 +788,7 @@ def translate_masks(modifiers: list[str]) -> int:
         try:
             masks.append(ModMasks[i.lower()])
         except KeyError as e:
-            raise XCBQError("Unknown modifier: %s" % i) from e
+            raise XCBQError(f"Unknown modifier: {i}") from e
     if masks:
         return functools.reduce(operator.or_, masks)
     else:
